define(["exports", "core/github", "core/pubsubhub"], function (exports, _github, _pubsubhub) {
  "use strict";

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.name = undefined;
  exports.run = run;
  // Module core/contrib
  // Fetches names of contributors from github and uses them to fill
  // in the content of elements with key identifiers:
  // #gh-commenters: people having contributed comments to issues.
  // #gh-contributors: people whose PR have been merged.
  // Spec editors get filtered out automatically.

  const name = exports.name = "core/contrib";

  function prop(prop) {
    return o => o[prop];
  }

  function findUsers(...thingsToFind) {
    const users = new Set();
    for (const things of thingsToFind) {
      for (const thing of things) {
        if (thing.user) {
          users.add(thing.user.url);
        }
      }
    }
    return [...users];
  }

  function join(things) {
    if (!things.length) {
      return "";
    }
    const { length } = things;
    const last = things[length - 1];
    switch (length) {
      case 1:
        return last;
      case 2:
        return `${things[0]} and ${last}`;
      default:
        return `${things.join(", ")}, and ${last}`;
    }
  }

  async function toHTML(urls, editors, element) {
    const args = await Promise.all(urls.map(_github.fetch));
    const names = args.map(([user]) => user.name || user.login).filter(name => !editors.includes(name)).sort((a, b) => a.toLowerCase().localeCompare(b.toLowerCase()));
    element.textContent = join(names);
    element.id = null;
  }

  async function run(conf) {
    const ghCommenters = document.getElementById("gh-commenters");
    const ghContributors = document.getElementById("gh-contributors");

    if (!ghCommenters && !ghContributors) {
      return;
    }

    if (!conf.githubAPI) {
      const elements = [];
      if (ghCommenters) elements.push("#" + ghCommenters.id);
      if (ghContributors) elements.push("#" + ghContributors.id);
      (0, _pubsubhub.pub)("error", `Requested list of contributors and/or commenters from GitHub (${elements.join(" and ")}) but config.githubAPI is not set.`);
      return;
    }

    const json = await (0, _github.fetch)(conf.githubAPI);
    const [issues, comments, contributors] = await Promise.all([(0, _github.fetchIndex)(json.issues_url), (0, _github.fetchIndex)(json.issue_comment_url), (0, _github.fetchIndex)(json.contributors_url)]);
    const editors = respecConfig.editors.map(prop("name"));
    const commenterUrls = findUsers(issues, comments);
    const contributorUrls = contributors.map(prop("url"));
    try {
      await Promise.all(toHTML(commenterUrls, editors, ghCommenters), toHTML(contributorUrls, editors, ghContributors));
    } catch (error) {
      (0, _pubsubhub.pub)("error", "Error loading contributors and/or commenters from  Error: " + error);
    }
  }
});
//# sourceMappingURL=contrib.js.map